from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View
from Products.WeatherReport.Locations import radar_map
from Products.WeatherReport.ReportCache import ReportCache, DEFAULT_TIMEOUT
from Products.WeatherReport import pymetar
from string import upper

WeatherReportSchema = BaseSchema + Schema((
    StringField('id',
                required=1,
                mode="rw",
                accessor="getId",
                mutator="setId",
                default=None,
                widget=IdWidget(label = 'Weather Report ID',
                                label_msgid="label_name",
                                description_msgid="help_name",
                                i18n_domain="plone"),
                ),

    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label = 'Weather Report Name',
                                    label_msgid="label_title",
                                    description = 'Name of the city'),
                                    i18n_domain="plone"),
                               
    StringField('station_id',
                searchable=1,
                required=1,
                widget=StringWidget(label = 'METAR Station ID'),
                ),
    ))



_cache = None

def getCache(timeout=DEFAULT_TIMEOUT):
    global _cache
    if _cache is None:
        _cache = ReportCache(timeout)
    return _cache


def _getWeatherInfo(self, weather_info_cat):
    """Get the weather_info_cat from the weather report.  We check to see if
       it has been cached first.  If it has, then we just return the piece 
       of information requested.  Otherwise we get all the info and cache
       it.  This caching speeds up the site and is nice to noaa.gov by not
       getting the information for every WeatherReport object everytime
       a page template is rendered"""
    
    global cache
    cache = getCache()
    no_data = 'no data'
    station_id = self.station_id
    reportinfo = cache.getReportInfo()
    
    # If we have cached info, check to make sure that it's real (not no_data)
    # and then return it.  If the cached data is no_data then we try to grab
    # the weather report again.
    #    
    if reportinfo is not None:
        station_cached_data = reportinfo.get(station_id)
        if station_cached_data is not None:
            weather_info = station_cached_data.get(weather_info_cat)
            if not (weather_info == no_data):
                return weather_info

    reportinfo = {}
    parsed_report = getParsedReport(self)
    
    if parsed_report is not None:
        dewpoint_c = parsed_report.getDewPointCelsius()
        dewpoint_f = parsed_report.getDewPointFahrenheit()
        humidity = parsed_report.getHumidity()
        temp_c = parsed_report.getTemperatureCelsius()
        temp_f = parsed_report.getTemperatureFahrenheit()
        sky_cond = parsed_report.getSkyConditions()
        weather = parsed_report.getWeather()
        wind_compass = parsed_report.getWindCompass()
        wind_speed_mph = parsed_report.getWindSpeedMilesPerHour()
        wind_speed_mps = parsed_report.getWindSpeed()
    else:
        dewpoint_c = no_data
        dewpoint_f = no_data
        humidity = no_data
        temp_c = no_data
        temp_f = no_data
        sky_cond = no_data
        weather = no_data
        wind_compass = no_data
        wind_speed_mph = no_data
        wind_speed_mps = no_data
    	
    reportinfo[station_id] = {'dewpoint_c': dewpoint_c,
                              'dewpoint_f': dewpoint_f,
                              'humidity': humidity,
                              'temp_c': temp_c,
                              'temp_f': temp_f, 
                              'sky_cond': sky_cond,
                              'weather': weather,
                              'wind_compass': wind_compass,
                              'wind_speed_mph': wind_speed_mph,
                              'wind_speed_mps': wind_speed_mps}
                              
    cache.cacheReportInfo(reportinfo)
    
    ### FIXME: There's gotta be a better way to do this
    ###        
    if weather_info_cat == 'dewpoint_c': 
    	return dewpoint_c    	
    elif weather_info_cat == 'dewpoint_f':
        return dewpoint_f
    elif weather_info_cat == 'humidity':
        return humidity
    elif weather_info_cat == 'temp_c':
        return temp_c
    elif weather_info_cat == 'temp_f':
        return temp_f
    elif weather_info_cat == 'sky_cond':
        return sky_cond
    elif weather_info_cat == 'weather':
        return weather
    elif weather_info_cat == 'wind_compass':
        return wind_compass
    elif weather_info_cat == 'wind_speed_mph':
        return wind_speed_mph
    elif weather_info_cat == 'wind_speed_mps':
        return wind_speed_mps

        

security = ClassSecurityInfo()
security.declareProtected(View, 'getParsedReport')


def getParsedReport(self):
    """return a parsed METAR report"""
    
    station_id = self.station_id
    try:
        rf = pymetar.ReportFetcher(station_id)
        rep = rf.FetchReport()  	
        rp = pymetar.ReportParser()
        pr = rp.ParseReport(rep)        
    except:
    	pr = None
    return pr


class WeatherReport(BaseContent):
    """Weather Report content type.  The type contains information
       that can be used to grab weather information for a given
       METAR station ID and then our page templates can be used to 
       display it."""
       
    schema = WeatherReportSchema
    
    def getDewC(self):
    	"""Get the Celsius dewpoint temperature"""
    	return _getWeatherInfo(self, 'dewpoint_c')
        
    def getDewF(self):
    	"""Get the Fahrenheit dewpoint temperature"""
    	return _getWeatherInfo(self, 'dewpoint_f')
        
    def getHumidity(self):
    	"""Get the relative humidity in percent"""
    	return _getWeatherInfo(self, 'humidity')
        
    def getTempC(self):
    	"""Get the Celsius temperature for the given station"""    	
    	return _getWeatherInfo(self, 'temp_c')

    def getTempF(self):
    	"""Get the Fahrenheit temperature for the given station"""    	
    	return _getWeatherInfo(self, 'temp_f')

    def getSkyConditions(self):
    	"""Get current sky conditions for the given station"""
    	return _getWeatherInfo(self, 'sky_cond')
    	
    def getWeather(self):
    	"""Get current weather (clear, rain, mist, snow, etc.)"""
    	return _getWeatherInfo(self, 'weather')
    	
    def getWindCompass(self):
    	"""Get wind compass direction"""
    	return _getWeatherInfo(self, 'wind_compass')
    	
    def getWindSpeedMPH(self):
    	"""Get the wind speed in miles per hour"""
    	return _getWeatherInfo(self, 'wind_speed_mph')

    def getWindSpeedMPS(self):
    	"""Get the wind speed in meters per second"""
    	return _getWeatherInfo(self, 'wind_speed_mps')
    	
    def getRadarImageURL(self):
    	radar_id = None
    	radar_id = radar_map.get(upper(self.station_id))
    	if radar_id is None:
    	    radar_imageURL = "http://image.weather.com/images/maps/current/curwx_600x405.jpg"
    	else:
	    radar_imageURL = "http://image.weather.com/web/radar/us_" + radar_id + "_closeradar_medium_usen.jpg"
	return radar_imageURL

    actions = ({
        'id': 'view',
        'name': 'View',
        'action': 'weather_view',
        'permissions': (CMFCorePermissions.View,)
        },)
   
registerType(WeatherReport)